pub type CvfsResult<O> = Result<O, CvfsErrors>;

#[derive(Debug)]
pub enum CvfsErrors {
    KeyManagementError(multi_key_manager::errors::KeyManagementError),
    GeneralIO(String),
    /// A constant time error occured in the `ring` crate
    GeneralRingEncryptionError,
}

impl From<std::io::Error> for CvfsErrors {
    fn from(e: std::io::Error) -> Self {
        CvfsErrors::GeneralIO(format!("{e:?}"))
    }
}

impl From<multi_key_manager::errors::KeyManagementError> for CvfsErrors {
    fn from(e: multi_key_manager::errors::KeyManagementError) -> Self {
        CvfsErrors::KeyManagementError(e)
    }
}

impl From<ring::error::Unspecified> for CvfsErrors {
    fn from(_: ring::error::Unspecified) -> Self {
        CvfsErrors::GeneralRingEncryptionError
    }
}
